/**
 * @file grid.c
 * @brief Source responsible for converting a point list to an ordered grid
 * @author Lukas Hostettler
 * @version 1.0
 * @date 2015-05-20
 */

#ifdef __cplusplus
extern "C" {
#endif

//needs replacing sometime...
#include <stdlib.h>
#include <assert.h>

#include "config.h"
#include "compiler.h"
#include "decoder.h"
#include "point.h"
#include "grid.h"

static int xOffsetList[MAX_NUM_POINTS];
static int yOffsetList[MAX_NUM_POINTS];
static IntPoint gridCoordinateList[MAX_NUM_POINTS];

//static inline int abs(int x){
//    return x>0?x:-x;
//}

//TODO: CHOOSE BETTER NAME FOR FUNCTION
/// @brief Finds the closest existing element larger than zero in an intGrid
/// @var    g   IntGrid structure
/// @var    x   Pointer for returning the x index of the found element in g, also serves as initial input of the spiral
/// @var    y   Pointer for returning the y index of the found element in g, also serves as initial input of the spiral
/// @returns 1 for success, 0 for faillure
int iterateSpiral(IntGrid g, int* x, int* y){
    int xdir = 0;
    int ydir = 1;
    int tmp;
    int counter = 0;
    int nextCorner = 0;
    int lengthChanger = 1;
    int length = 0;
    while(g.array[*x][*y] < 0){
        if(counter == nextCorner){
            tmp = ydir;
            ydir = xdir;
            xdir = -tmp;
            if(lengthChanger){
                length += 1;
                lengthChanger = 0;
            }
            else
                lengthChanger = 1;
            nextCorner += length;
        }
        counter++;
        *x += xdir;
        *y += ydir;
        if(*x >= g.numCols || *x < 0 || *y < 0 || *y >= g.numRows)
            return 0;
    }
    return 1;
}

/*
void testSpiral(){
    IntGrid g=intGridCreate(5,5);
    int i;
    for(i=0;i<25;++i){
        g.data[i]=i-25;
    }
    int a=2;
    int b=2;
    iterateSpiral(g,&a,&b);
}
*/

int correctMeanLength(IntPoint* pointArray, IntPoint* means, int power, int n, IntPoint* origin){

    //swap vectors if order is bad
    if(means[0].x*means[1].y - means[0].y*means[1].x < 0){
        IntPoint tmp = means[1];
        means[1] = means[0];
        means[0] = tmp;
    }

    int i, j, xoffset, yoffset;
    int minGridX = INT32_MAX, maxGridX = INT32_MIN, minGridY = INT32_MAX, maxGridY = INT32_MIN;
    int a = means[0].x;// Matrix elements of matrix
    int b = means[1].x;// a b
    int c = means[0].y;// c d
    int d = means[1].y;//
    int determinant = a*d - c*b;
    if(!determinant) //skip if determinant==0;
        return 0;
    IntPoint gridCoordinate[n]; //List of coordinates
    IntPoint projectedArray[n]; //List of projections
    IntPoint* gridCoordinatePointer = gridCoordinate+n;
    IntPoint* projectedArrayPtr = projectedArray+n;
    IntPoint* pointPointer = pointArray+n;

    // Transform to new coordinate system
    while(pointPointer != pointArray){
        --pointPointer;
        --projectedArrayPtr;
        --gridCoordinatePointer;

        //project
        xoffset = pointPointer->x - origin->x;
        yoffset = pointPointer->y - origin->y;
        projectedArrayPtr->x = (((int64_t)d*xoffset - b*yoffset) << power)/determinant;
        projectedArrayPtr->y = (((int64_t)-c*xoffset + a*yoffset) << power)/determinant;

        //assign coordinate
        gridCoordinatePointer->x = (projectedArrayPtr->x >> power) + ((projectedArrayPtr->x >> (power - 1))&1);
        if(gridCoordinatePointer->x < minGridX)
            minGridX = gridCoordinatePointer->x;
        if(gridCoordinatePointer->x > maxGridX)
            maxGridX = gridCoordinatePointer->x;
        gridCoordinatePointer->y = (projectedArrayPtr->y >> power) + ((projectedArrayPtr->y >> (power - 1))&1);
        if(gridCoordinatePointer->y < minGridY)
            minGridY = gridCoordinatePointer->y;
        if(gridCoordinatePointer->y > maxGridY)
            maxGridY = gridCoordinatePointer->y;
    }

    //Check if the points can fit in statically allocated Grid
    if(maxGridX - minGridX > MAX_GRID_LENGTH || maxGridY - minGridY > MAX_GRID_LENGTH)
        return 0;

    //Fill indice grid with found coordinates
    IntGrid indiceGrid = intGridCreate(maxGridX - minGridX + 1, maxGridY - minGridY + 1);
    intGridFill(indiceGrid, NOPOINT);
    for(i=0; i<n; ++i){
        indiceGrid.array[gridCoordinate[i].y - minGridY][gridCoordinate[i].x - minGridX] = i;
    }

    //Correct the mean length of the first mean
    int accumulator[n];
    int counter = 0;
    for(i=0; i<indiceGrid.numRows; ++i){
        for(j=0; j<indiceGrid.numCols - 1; ++j){
            int idx1 = indiceGrid.array[i][j];
            int idx2 = indiceGrid.array[i][j + 1];
            if(idx1 >= 0 && idx2 >= 0)
                accumulator[counter++] = projectedArrayPtr[idx2].x - projectedArrayPtr[idx1].x;
        }
    }
    if(counter){
        IntList tmp;
        tmp.list = accumulator;
        tmp.numElements = counter;
        int corr = intListMedian(tmp);
        means[0].x = (a*corr) >> power;
        means[0].y = (c*corr) >> power;
    }

    //Correct the mean length of the second mean
    counter = 0;
    for(i=0; i<indiceGrid.numRows - 1; ++i){
        for(j=0; j<indiceGrid.numCols; ++j){
            int idx1 = indiceGrid.array[i][j];
            int idx2 = indiceGrid.array[i + 1][j];
            if(idx1 >= 0 && idx2 >= 0)
                accumulator[counter++] = projectedArrayPtr[idx2].y - projectedArrayPtr[idx1].y;
        }
    }
    if(counter){
        IntList tmp;
        tmp.list = accumulator;
        tmp.numElements = counter;
        int corr = intListMedian(tmp);
        means[1].x = (b*corr) >> power;
        means[1].y = (d*corr) >> power;
    }

    //find a good central point for the origin
    int rowEst = indiceGrid.numRows/2;
    int colEst = indiceGrid.numCols/2;


    if(iterateSpiral(indiceGrid, &rowEst, &colEst)){
        *origin = pointArray[indiceGrid.array[rowEst][colEst]];
    }

    intGridFree(&indiceGrid);

    return 1;
}

///@brief   Calculates the median displacement between dots and their corresponding grid coordinates. Corrects the offset information.
///@var     d Completely established DotInformation
///@return  the median (x and y) of the offsets
IntPoint medianCorrectOffsets(DotInformation d){
    IntPoint median;
    IntList tmp, cpy;
    int i;

    tmp.numAlloc = d.numElements;
    tmp.numElements = d.numElements;

    tmp.list = d.xoffsets;
    cpy = intListCopy(&tmp); //used in order to not destroy the order of the list
    median.x = intListMedian(cpy);
    intListFree(&cpy);

    tmp.list = d.yoffsets;
    cpy = intListCopy(&tmp);
    median.y = intListMedian(cpy);
    intListFree(&cpy);
    for(i=0; i<d.numElements; ++i){
        d.xoffsets[i] -= median.x;
        d.yoffsets[i] -= median.y;
    }

    return median;
}

///@brief Creates an empty DotInformation
/// @return A new empty DotInformation
inline DotInformation dotInfoCreateEmpty(){
    DotInformation ans;

    ans.numElements = 0;
    ans.gridMaxCols = 0;
    ans.gridMaxRows = 0;
    ans.xoffsets = xOffsetList;
    ans.yoffsets = yOffsetList;
    ans.gridCoordinate = gridCoordinateList;

    return ans;
}

DotInformation dotInfoInit(IntPoint* pointArray, IntPoint* means, int power, int numPoints, IntPoint origin){
    DotInformation ans = dotInfoCreateEmpty();
    int xoffset, yoffset, xproj, yproj, minGridX = INT32_MAX, maxGridX = INT32_MIN, minGridY = INT32_MAX, maxGridY = INT32_MIN;
    IntPoint medianOffset;
    //matrix [a b ;c d];
    int a = means[0].x;
    int b = means[1].x;
    int c = means[0].y;
    int d = means[1].y;
    int determinant = a*d - c*b;
    if(!determinant || numPoints < 1)
        return ans;

    ans.numElements = numPoints;

    IntPoint* pointOrig = pointArray; //pointer trough pointArray;
    int* xoffsets = ans.xoffsets; // pointer through offsetArray
    int* yoffsets = ans.yoffsets; // pointer through offsetArray
    IntPoint* gridCoordinate = ans.gridCoordinate; //pointer trough gridCoordiates
    while(pointOrig <= &pointArray[numPoints - 1]){
        //project
        xoffset = pointOrig->x - origin.x;
        yoffset = pointOrig->y - origin.y;
        xproj = ((int64_t)(d*xoffset - b*yoffset) << power)/determinant;
        yproj = ((int64_t)(-c*xoffset + a*yoffset) << power)/determinant;

        //assign coordinate
        gridCoordinate->x = (xproj >> power) + ((xproj >> (power - 1))&1);
        gridCoordinate->y = (yproj >> power) + ((yproj >> (power - 1))&1);

        //update minMax
        if(gridCoordinate->x < minGridX)
            minGridX = gridCoordinate->x;
        if(gridCoordinate->x > maxGridX)
            maxGridX = gridCoordinate->x;
        if(gridCoordinate->y < minGridY)
            minGridY = gridCoordinate->y;
        if(gridCoordinate->y > maxGridY)
            maxGridY = gridCoordinate->y;

        //Debug set point to grid coordinate.
        pointOrig->x = a*gridCoordinate->x + b*gridCoordinate->y + origin.x;
        pointOrig->y = c*gridCoordinate->x + d*gridCoordinate->y + origin.y;

        //find offsets:
        *xoffsets = xproj - (gridCoordinate->x << power);
        *yoffsets = yproj - (gridCoordinate->y << power);

        //Increment pointers;
        ++pointOrig;
        ++xoffsets;
        ++yoffsets;
        ++gridCoordinate;
    }
    ans.gridMaxRows = maxGridX - minGridX + 1;
    ans.gridMaxCols = maxGridY - minGridY + 1;

    //Shift coordinates to be positive.
    for(gridCoordinate = ans.gridCoordinate; gridCoordinate < ans.gridCoordinate + ans.numElements; ++gridCoordinate){
        intPointAdd(gridCoordinate, -minGridX, -minGridY);
    }

    medianOffset = medianCorrectOffsets(ans);

    while(pointOrig != pointArray){
        pointOrig--;
        pointOrig->x += (a*medianOffset.x + b*medianOffset.y) >> power;
        pointOrig->y += (c*medianOffset.x + d*medianOffset.y) >> power;
    }
    ans.gridOrigin = origin;
    intPointAdd(&(ans.gridOrigin), a*minGridX + b*minGridY, c*minGridX + d*minGridY);
    intPointAdd(&(ans.gridOrigin), (a*medianOffset.x + b*medianOffset.y) >> power, (c*medianOffset.x + d*medianOffset.y) >> power);

    return ans;
}

ProbabilityGrids makeProbabilityGrids(DotInformation dotInfo){
    ProbabilityGrids ans;
    IntPoint* probs = probabilities(dotInfo.xoffsets, dotInfo.yoffsets, dotInfo.numElements);
    ans.prob1 = intGridCreate(dotInfo.gridMaxCols, dotInfo.gridMaxRows);
    intGridFill(ans.prob1, 0);
    ans.prob2 = intGridCreate(dotInfo.gridMaxCols, dotInfo.gridMaxRows);
    intGridFill(ans.prob2, 0);
    ans.maxProb = intGridCreate(dotInfo.gridMaxCols, dotInfo.gridMaxRows);
    intGridFill(ans.maxProb, 0);

    int probProduct, i;
    for(i=0; i<dotInfo.numElements; ++i){
        probProduct = abs(probs[i].x*probs[i].y);
        IntPoint coord = dotInfo.gridCoordinate[i];
        if(probProduct >= ans.maxProb.array[coord.x][coord.y]){
            ans.maxProb.array[coord.x][coord.y] = probProduct;
            ans.prob1.array[coord.x][coord.y] = probs[i].x;
            ans.prob2.array[coord.x][coord.y] = probs[i].y;
        }
    }
    free(probs); //TODO: GET RID OF THIS
    //temporary

    return ans;
}

//TODO: THIS LOOKS EXTREMELY INEFFICIENT
void probabilityGridsTurn(ProbabilityGrids* grids, int numRightTurns){
    numRightTurns %= 4;
    IntGrid tmpGrid;

    switch(numRightTurns){
    case 0:
        break;
    case 3:
        tmpGrid = intGridTurn(grids->prob1, numRightTurns, -1);
        intGridFree(&grids->prob1); //no memory leaks
        grids->prob1 = intGridTurn(grids->prob2, numRightTurns, 1);
        intGridFree(&grids->prob2);
        grids->prob2 = tmpGrid;
        break;
    case 2:
        tmpGrid = intGridTurn(grids->prob1, numRightTurns, -1);
        intGridFree(&grids->prob1);
        grids->prob1 = tmpGrid;
        tmpGrid = intGridTurn(grids->prob2, numRightTurns, -1);
        intGridFree(&grids->prob2);
        grids->prob2 = tmpGrid;
        break;
    case 1:
        tmpGrid = intGridTurn(grids->prob1, numRightTurns, 1);
        intGridFree(&grids->prob1);
        grids->prob1 = intGridTurn(grids->prob2, numRightTurns, -1);
        intGridFree(&grids->prob2);
        grids->prob2 = tmpGrid;
        break;
    }
}

//TODO: THERE MUST BE A BETTER WAY OF DOING THIS BY NOT ALLOCING AND FREEING MATRICES ALL OVER THE PLACE
/// @brief  returns a new intGrid which is turned numRight Turns and multiplied by factor
/// @var    grid            the intGrid to be turned
/// @var    numRightTurns   how many 90°-turns have to be made [0-3]
/// @var    factor          a multiplicative factor for all the intgrids entries
/// @return a turned version of grid
IntGrid intGridTurn(IntGrid grid, int numRightTurns, int factor){
    int i,j;
    numRightTurns %= 4;
    IntGrid tmpGrid = intGridCreateEmpty();

    switch(numRightTurns){
    case 0:
        break;
    case 1:
        tmpGrid = intGridCreate(grid.numRows, grid.numCols);
        for(i=0; i<grid.numCols; ++i){
            for(j=0; j<grid.numRows; ++j){
                tmpGrid.array[i][j] = factor*grid.array[grid.numRows - 1 - j][i];
            }
        }
        break;
    case 2:
        tmpGrid = intGridCreate(grid.numCols, grid.numRows);
        for(i=0; i<grid.numRows; ++i){
            for(j=0; j<grid.numCols; ++j){
                tmpGrid.array[i][j] = factor*grid.array[grid.numRows - 1 - i][grid.numCols - 1 - j];
            }
        }
        break;
    case 3:
        tmpGrid = intGridCreate(grid.numRows, grid.numCols);
        for(i=0; i<grid.numCols; ++i){
            for(j=0; j<grid.numRows; ++j){
                tmpGrid.array[i][j] = factor*grid.array[j][grid.numCols - 1 - i];
            }
        }
        break;
    }

    return tmpGrid;
}

void probabilityGridsFree(ProbabilityGrids * grid){
    intGridFree(&(grid->maxProb));
    intGridFree(&grid->prob1);
    intGridFree(&grid->prob2);
}

#ifdef __cplusplus
}
#endif

