/**
 * @file intgrid.h
 * @brief TODO
 * @author Lukas Hostettler
 * @version 1.0
 * @date TODO
 */

#ifndef INTGRID_H

#ifdef __cplusplus
extern "C" {
#endif

#include "list.h"
#include "neighbours.h"


/// Symbol for a missing point
#define NOPOINT -1

typedef struct _IntGrid{
    int numCols;
    int numRows;
    int *data;   //!<The actual pointer to data.
    int **array; //! Pointers construct for easy access: IntGrid[numRow][numCol];
} IntGrid; //! a dynamic array


///@brief   Creates an empty 0x0 IntGrid. There is no data allocation.
///@returns empty IntGrid Structure
IntGrid intGridCreateEmpty();

///@brief   Creates an empty IntGrid of size numCol x numRow.
///@var     numCol      number of columns of the grid
///@var     numRow      nomber of rows of the grid
///@return  an allocated IntGrid of size numCol x numRow
IntGrid intGridCreate(int numCol, int numRow);

///@brief   Deep Copy of an IntGrid
///@var     other       the IntGrid to copy
///@return  a copy of other
IntGrid intGridCopy(IntGrid other);

///@brief   Deep Copy of an region of intrest of an IntGrid
///@var     other       the IntGrid to copy
///@var     topRow      index of first row to copy
///@var     leftCol     index of first columns to copy
///@var     numRows     number of rows to copy
///@var     numCols     number of columns to copy
///@return  a copy of the specified Region of interes of other
IntGrid intGridCopySub(IntGrid other, int topRow, int leftCol, int numRows, int numCols);

///@brief   Adds Elementwise a+b. Raises Error when dimensions do not correspond.
///@var     a           an Intgrid to sum
///@var     b           an Intgrid to sum
///@return  the elementwise sum of a and b
IntGrid intGridAdd(IntGrid a, IntGrid b);

///@brief   Find Elementwise maximum. Raises error when dimensions disagree.
///@var     a           an Intgrid
///@var     b           another Intgrid
///@return  A new IntGrid with the bigger entries chosen from a and b
IntGrid intGridMax(IntGrid a, IntGrid b);

//TODO: REMOVE as it is unused
///@brief   Generic Function to apply functions to every element of g
///@var     g           the IntGrid to apply the function to
///@var     f           function pointer representing the elementwise operation
void intGridApplyToEveryElement(IntGrid g, int(*f)(int));

///@brief   makes a convolution with a square window of size windowsize and coefficient 1
///@var     g           the IntGrid to be convoluted
///@var     windowSize  width and hight of the convolution kernel
///@return  An Intgrid containing the entries of the inner convolution. Its size is g.numRows-windowSize+1 x g.numCols-windowsSize+1
IntGrid intGridLocalSum(IntGrid g, int windowSize);

///@brief   finds index of top,left corner of the sub square of size n x n with the hightest sum of values
///@var     grid        IntGrid to be examined
///@var     nCol        pointer to an integer. Recieves leftmost column index of the highest sum of values square
///@var     nRow        pointer to an integer. Recieves topmost row index of the highest sum of values square
///@var     n           sidelength of the subsquare
void intGridFindBestNxN(IntGrid grid, int* nCol, int* nRow, int n);

///@brief   Turns the intGrid
///@var     grid            the Intgrid to be turned
///@var     numRightTurns   number of turns to be done
///@var     factor          multiplicative factor applied to all entries (for inversing the probabilities)
///@return  A turned version of the given grid
IntGrid intGridTurn(IntGrid grid, int numRightTurns, int factor);


///@brief   Frees the allocated memory of an IntGrid.
///@var     g           pointer to an Intgrid
void intGridFree(IntGrid * g);

//TODO: REMOVE
///@brief   Test function for allocation, processing and freeing of intgrids
void intGridTest();

#ifdef __cplusplus
}
#endif

#endif //INTGRID_H

