/**
 * @file list.h
 * @brief TODO
 * @author Lukas Hostettler
 * @version 1.0
 * @date TODO
 */

#ifndef LIST_H
#define LIST_H

//TODO: MOVE DEPENDENCY TO list.c
#include "imgsegment.h"

#ifdef __cplusplus
extern "C" {
#endif

///@brief   Dynamic list of integers
typedef struct {
    int numElements;//! number of elements in the list
    int numAlloc;   //! number of allocated places for list
    int * list;     //! pointer to beginning of list
} IntList;

///@brief   Constructor of an IntList
///@returns Empty IntList
IntList intListCreate();

///@brief   Deep copy of an IntList
///@var     dyn     Pointer to list to be copied
///@return  A deep copy of intlist
IntList intListCopy(IntList* dyn);


///@brief   Add an element at the end of the list
///@var     dyn     Pointer to the IntList to be modified
///@var     Element the integer to be appended to the end of the list
void intListPush(IntList* dyn, int element);

///@brief   Gives pointer to the n-th element of dyn
///@var     dyn     Pointer to the IntList
///@var     n       Index of the element to be returned
///@returns Adress of n-th element of the list
int* intListGetPointer(IntList* dyn,int n);


///@brief   Shrinks the allocated space of dyn to fit its number of elements
///@var     dyn     Pointer to the IntList to be shrinked
void intListShrink(IntList* dyn);

///@brief   Gets address of first element of the list
///@var     dyn     Pointer to the IntList
///@return  Adress of the first element of the internal int-array
int* intListGetStart(IntList* dyn);

///@brief   Gets address of last element +1 of the list
///@var     dyn     Pointer to the IntList
///@return  Adress of the first element not defined element of the internal int-array
int* intListGetEnd(IntList* dyn);

///@brief   Frees the internal array of dyn
///@var     dyn     Pointer to the IntList to be freed
void intListFree (IntList* dyn);

#ifdef __cplusplus
}
#endif

#endif // LIST_H

